@@ -45,6 +45,8 @@
 #  define safe_fseek fseek
 #endif
 
+static inline void IGUR_sizet(size_t ignored) { (void)ignored; }  /* Ignore GCC Unused Result */
+
 /************************************************************************/
 /*                             SfRealloc()                              */
 /*                                                                      */
@@ -54,10 +56,7 @@
 static void * SfRealloc( void * pMem, int nNewSize )
 
 {
-  if( pMem == NULL )
-    return( (void *) malloc(nNewSize) );
-  else
-    return( (void *) realloc(pMem,nNewSize) );
+  return( (void *) realloc(pMem,nNewSize) );
 }
 
 /************************************************************************/
@@ -183,8 +182,11 @@ DBFHandle msDBFOpen( const char * pszFilename, const char * pszAccess )
       psDBF->fp = fopen( pszDBFFilename, pszAccess );
     }
   }
-  if( psDBF->fp == NULL )
+  if( psDBF->fp == NULL ) {
+    msFree(pszDBFFilename);
+    msFree(psDBF);
     return( NULL );
+  }
 
   psDBF->bNoHeader = MS_FALSE;
   psDBF->nCurrentRecord = -1;
@@ -199,7 +201,12 @@ DBFHandle msDBFOpen( const char * pszFilename, const char * pszAccess )
   /*  Read Table Header info                                              */
   /* -------------------------------------------------------------------- */
   pabyBuf = (uchar *) msSmallMalloc(500);
-  fread( pabyBuf, 32, 1, psDBF->fp );
+  if( fread( pabyBuf, 32, 1, psDBF->fp ) != 1 )
+  {
+    msFree(psDBF);
+    msFree(pabyBuf);
+    return( NULL );
+  }
 
   psDBF->nRecords = nRecords =
                       pabyBuf[4] + pabyBuf[5]*256 + pabyBuf[6]*256*256 + pabyBuf[7]*256*256*256;
@@ -218,7 +225,13 @@ DBFHandle msDBFOpen( const char * pszFilename, const char * pszAccess )
   psDBF->pszHeader = (char *) pabyBuf;
 
   fseek( psDBF->fp, 32, 0 );
-  fread( pabyBuf, nHeadLen, 1, psDBF->fp );
+  if( fread( pabyBuf, nHeadLen - 32, 1, psDBF->fp ) != 1 )
+  {
+    msFree(psDBF->pszCurrentRecord);
+    msFree(psDBF);
+    msFree(pabyBuf);
+    return( NULL );
+  }
 
   psDBF->panFieldOffset = (int *) msSmallMalloc(sizeof(int) * nFields);
   psDBF->panFieldSize = (int *) msSmallMalloc(sizeof(int) * nFields);
@@ -271,7 +284,7 @@ void  msDBFClose(DBFHandle psDBF)
     uchar   abyFileHeader[32];
 
     fseek( psDBF->fp, 0, 0 );
-    fread( abyFileHeader, 32, 1, psDBF->fp );
+    IGUR_sizet(fread( abyFileHeader, 32, 1, psDBF->fp ));
 
     abyFileHeader[1] = 95;      /* YY */
     abyFileHeader[2] = 7;     /* MM */
@@ -301,7 +314,7 @@ void  msDBFClose(DBFHandle psDBF)
   free( psDBF->pszHeader );
   free( psDBF->pszCurrentRecord );
 
-  if(psDBF->pszStringField) free(psDBF->pszStringField);
+  free(psDBF->pszStringField);
 
   free( psDBF );
 }
@@ -466,7 +479,7 @@ int msDBFAddField(DBFHandle psDBF, const char * pszFieldName, DBFFieldType eType
 /*      Based on DBFIsAttributeNULL of shapelib                         */
 /************************************************************************/
 
-int DBFIsValueNULL( const char* pszValue, char type )
+static int DBFIsValueNULL( const char* pszValue, char type )
 
 {
   switch(type) {
@@ -494,13 +507,13 @@ int DBFIsValueNULL( const char* pszValue, char type )
 /*                                                                      */
 /*      Read one of the attribute fields of a record.                   */
 /************************************************************************/
-static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
+static const char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
 
 {
   int         i;
   unsigned int nRecordOffset;
-  uchar *pabyRec;
-  char  *pReturnField = NULL;
+  const uchar *pabyRec;
+  const char  *pReturnField = NULL;
 
   /* -------------------------------------------------------------------- */
   /*  Is the request valid?                             */
@@ -524,12 +537,16 @@ static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
     nRecordOffset = psDBF->nRecordLength * hEntity + psDBF->nHeaderLength;
 
     safe_fseek( psDBF->fp, nRecordOffset, 0 );
-    fread( psDBF->pszCurrentRecord, psDBF->nRecordLength, 1, psDBF->fp );
+    if( fread( psDBF->pszCurrentRecord, psDBF->nRecordLength, 1, psDBF->fp ) != 1 )
+    {
+      msSetError(MS_DBFERR, "Cannot read record %d.", "msDBFReadAttribute()",hEntity );
+      return( NULL );
+    }
 
     psDBF->nCurrentRecord = hEntity;
   }
 
-  pabyRec = (uchar *) psDBF->pszCurrentRecord;
+  pabyRec = (const uchar *) psDBF->pszCurrentRecord;
   /* DEBUG */
   /* printf("CurrentRecord(%c):%s\n", psDBF->pachFieldType[iField], pabyRec); */
 
@@ -544,7 +561,7 @@ static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
   /* -------------------------------------------------------------------- */
   /*  Extract the requested field.              */
   /* -------------------------------------------------------------------- */
-  strncpy( psDBF->pszStringField,(char *) pabyRec+psDBF->panFieldOffset[iField], psDBF->panFieldSize[iField] );
+  strncpy( psDBF->pszStringField,(const char *) pabyRec+psDBF->panFieldOffset[iField], psDBF->panFieldSize[iField] );
   psDBF->pszStringField[psDBF->panFieldSize[iField]] = '\0';
 
   /*
@@ -711,7 +728,8 @@ static int msDBFWriteAttribute(DBFHandle psDBF, int hEntity, int iField, void *
     nRecordOffset = psDBF->nRecordLength * hEntity + psDBF->nHeaderLength;
 
     safe_fseek( psDBF->fp, nRecordOffset, 0 );
-    fread( psDBF->pszCurrentRecord, psDBF->nRecordLength, 1, psDBF->fp );
+    if( fread( psDBF->pszCurrentRecord, psDBF->nRecordLength, 1, psDBF->fp ) != 1 )
+      return MS_FALSE;
 
     psDBF->nCurrentRecord = hEntity;
   }
